@@ -40,6 +40,14 @@ rstto_image_list_class_init(RsttoImageListClass *);
 static void
 rstto_image_list_dispose(GObject *object);
 
+static void
+cb_file_monitor_changed (
+        GFileMonitor      *monitor,
+        GFile             *file,
+        GFile             *other_file,
+        GFileMonitorEvent  event_type,
+        gpointer           user_data );
+
 static void 
 rstto_image_list_iter_init(RsttoImageListIter *);
 static void
@@ -47,6 +55,37 @@ rstto_image_list_iter_class_init(RsttoImageListIterClass *);
 static void
 rstto_image_list_iter_dispose(GObject *object);
 
+static void
+cb_rstto_wrap_images_changed (
+        GObject *settings,
+        GParamSpec *pspec,
+        gpointer user_data);
+
+static void
+rstto_image_list_monitor_dir (
+        RsttoImageList *image_list,
+        GFile *dir );
+
+static void
+rstto_image_list_remove_all (
+        RsttoImageList *image_list);
+
+static gboolean
+iter_next (
+        RsttoImageListIter *iter,
+        gboolean sticky);
+
+static gboolean
+iter_previous (
+        RsttoImageListIter *iter,
+        gboolean sticky);
+
+static void 
+iter_set_position (
+        RsttoImageListIter *iter,
+        gint pos,
+        gboolean sticky);
+
 static RsttoImageListIter * rstto_image_list_iter_new ();
 
 static gint
@@ -75,16 +114,25 @@ enum
 struct _RsttoImageListIterPriv
 {
     RsttoImageList *image_list;
-    RsttoFile *file;
+    RsttoFile      *file;
+    
+    /* This is set if the iter-position is chosen by the user */
+    gboolean        sticky; 
 };
 
 struct _RsttoImageListPriv
 {
-    GList *images;
-    gint n_images;
+    GFileMonitor *monitor;
+    RsttoSettings *settings;
+    GtkFileFilter *filter;
 
-    GSList *iterators;
-    GCompareFunc cb_rstto_image_list_compare_func;
+    GList        *images;
+    gint          n_images;
+
+    GSList       *iterators;
+    GCompareFunc  cb_rstto_image_list_compare_func;
+
+    gboolean      wrap_images;
 };
 
 static gint rstto_image_list_signals[RSTTO_IMAGE_LIST_SIGNAL_COUNT];
@@ -121,7 +169,23 @@ rstto_image_list_init(RsttoImageList *image_list)
 {
 
     image_list->priv = g_new0 (RsttoImageListPriv, 1);
+    image_list->priv->settings = rstto_settings_new ();
+    image_list->priv->filter = gtk_file_filter_new ();
+    g_object_ref_sink (image_list->priv->filter);
+    gtk_file_filter_add_pixbuf_formats (image_list->priv->filter);
+
     image_list->priv->cb_rstto_image_list_compare_func = (GCompareFunc)cb_rstto_image_list_image_name_compare_func;
+
+    image_list->priv->wrap_images = rstto_settings_get_boolean_property (
+            image_list->priv->settings,
+            "wrap-images");
+
+    g_signal_connect (
+            G_OBJECT(image_list->priv->settings),
+            "notify::wrap-images",
+            G_CALLBACK (cb_rstto_wrap_images_changed),
+            image_list);
+
 }
 
 static void
@@ -172,7 +236,24 @@ rstto_image_list_class_init(RsttoImageListClass *nav_class)
 static void
 rstto_image_list_dispose(GObject *object)
 {
-    /*RsttoImageList *image_list = RSTTO_IMAGE_LIST(object);*/
+    RsttoImageList *image_list = RSTTO_IMAGE_LIST(object);
+    if (NULL != image_list->priv)
+    {
+        if (image_list->priv->settings)
+        {
+            g_object_unref (image_list->priv->settings);
+            image_list->priv->settings = NULL;
+        }
+
+        if (image_list->priv->filter)
+        {
+            g_object_unref (image_list->priv->filter);
+            image_list->priv->filter= NULL;
+        }
+        
+        g_free (image_list->priv);
+        image_list->priv = NULL;
+    }
 }
 
 RsttoImageList *
@@ -186,33 +267,54 @@ rstto_image_list_new (void)
 }
 
 gboolean
-rstto_image_list_add_file (RsttoImageList *image_list, RsttoFile *file, GError **error)
+rstto_image_list_add_file (
+        RsttoImageList *image_list,
+        RsttoFile *file,
+        GError **error )
 {
+    GtkFileFilterInfo filter_info;
     GList *image_iter = g_list_find (image_list->priv->images, file);
+    GSList *iter = image_list->priv->iterators;
+
+    g_return_val_if_fail ( NULL != file , FALSE);
+    g_return_val_if_fail ( RSTTO_IS_FILE (file) , FALSE);
 
     if (!image_iter)
     {
         if (file)
         {
-            image_list->priv->images = g_list_insert_sorted (image_list->priv->images, file, rstto_image_list_get_compare_func (image_list));
+            filter_info.contains =  GTK_FILE_FILTER_MIME_TYPE | GTK_FILE_FILTER_URI;
+            filter_info.uri = rstto_file_get_uri (file);
+            filter_info.mime_type = rstto_file_get_content_type (file);
 
-            image_list->priv->n_images++;
-
-            g_signal_emit (G_OBJECT (image_list), rstto_image_list_signals[RSTTO_IMAGE_LIST_SIGNAL_NEW_IMAGE], 0, file, NULL);
-            if (image_list->priv->n_images == 1)
+            if ( TRUE == gtk_file_filter_filter (image_list->priv->filter, &filter_info))
             {
+                g_object_ref (G_OBJECT (file));
+
+                image_list->priv->images = g_list_insert_sorted (image_list->priv->images, file, rstto_image_list_get_compare_func (image_list));
+
+                image_list->priv->n_images++;
+
+                g_signal_emit (G_OBJECT (image_list), rstto_image_list_signals[RSTTO_IMAGE_LIST_SIGNAL_NEW_IMAGE], 0, file, NULL);
                 /** TODO: update all iterators */
-                GSList *iter = image_list->priv->iterators;
                 while (iter)
                 {
-                    g_signal_emit (G_OBJECT (iter->data), rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_CHANGED], 0, NULL);
+                    if (FALSE == RSTTO_IMAGE_LIST_ITER(iter->data)->priv->sticky)
+                    {
+                        rstto_image_list_iter_find_file (iter->data, file);
+                    }
                     iter = g_slist_next (iter);
                 }
+                return TRUE;
+            }
+            else
+            {
+                return FALSE;
             }
-            return TRUE;
         }
         return FALSE;
     }
+
     g_signal_emit (G_OBJECT (image_list), rstto_image_list_signals[RSTTO_IMAGE_LIST_SIGNAL_NEW_IMAGE], 0, image_iter->data, NULL);
 
     return TRUE;
@@ -264,11 +366,11 @@ rstto_image_list_remove_file (RsttoImageList *image_list, RsttoFile *file)
             {
                 if (rstto_image_list_iter_get_position (iter->data) == rstto_image_list_get_n_images (image_list)-1)
                 {
-                    rstto_image_list_iter_previous (iter->data);
+                    iter_previous (iter->data, FALSE);
                 }
                 else
                 {
-                    rstto_image_list_iter_next (iter->data);
+                    iter_next (iter->data, FALSE);
                 }
                 /* If the image is still the same, 
                  * it's a single item list,
@@ -292,7 +394,7 @@ rstto_image_list_remove_file (RsttoImageList *image_list, RsttoFile *file)
             {
                 if (rstto_file_equal(afile, file))
                 {
-                    rstto_image_list_iter_next (iter->data);
+                    iter_next (iter->data, FALSE);
                 }
             }
             iter = g_slist_next (iter);
@@ -303,7 +405,7 @@ rstto_image_list_remove_file (RsttoImageList *image_list, RsttoFile *file)
     }
 }
 
-void
+static void
 rstto_image_list_remove_all (RsttoImageList *image_list)
 {
     GSList *iter = NULL;
@@ -314,12 +416,129 @@ rstto_image_list_remove_all (RsttoImageList *image_list)
     iter = image_list->priv->iterators;
     while (iter)
     {
-        rstto_image_list_iter_set_position (iter->data, -1);
+        iter_set_position (iter->data, -1, FALSE);
         iter = g_slist_next (iter);
     }
     g_signal_emit (G_OBJECT (image_list), rstto_image_list_signals[RSTTO_IMAGE_LIST_SIGNAL_REMOVE_ALL], 0, NULL);
 }
 
+gboolean
+rstto_image_list_set_directory (
+        RsttoImageList *image_list,
+        GFile *dir,
+        GError **error )
+{
+    /* Declare variables */
+    GFileEnumerator *file_enumerator = NULL;
+    GFileInfo *file_info;
+    const gchar *filename;
+    const gchar *content_type;
+    GFile *child_file;
+
+    /* Source code block */
+    rstto_image_list_remove_all (image_list);
+
+    /* Allow all images to be removed by providing NULL to dir */
+    if ( NULL != dir )
+    {
+        file_enumerator = g_file_enumerate_children (dir, "standard::*", 0, NULL, NULL);
+
+        if (NULL != file_enumerator)
+        {
+            for(file_info = g_file_enumerator_next_file (file_enumerator, NULL, NULL);
+                NULL != file_info;
+                file_info = g_file_enumerator_next_file (file_enumerator, NULL, NULL))
+            {
+                filename = g_file_info_get_name (file_info);
+                content_type  = g_file_info_get_content_type (file_info);
+                child_file = g_file_get_child (dir, filename);
+                if (strncmp (content_type, "image/", 6) == 0)
+                {
+                    rstto_image_list_add_file (image_list, rstto_file_new (child_file), NULL);
+                }
+            }
+            g_object_unref (file_enumerator);
+            file_enumerator = NULL;
+        }
+    }
+
+    rstto_image_list_monitor_dir ( image_list, dir );
+
+    return TRUE;
+}
+
+static void
+rstto_image_list_monitor_dir (
+        RsttoImageList *image_list,
+        GFile *dir )
+{
+    GFileMonitor *monitor = NULL;
+
+    if ( NULL != image_list->priv->monitor )
+    {
+        g_object_unref (image_list->priv->monitor);
+        image_list->priv->monitor = NULL;
+    }
+
+    /* Allow a monitor to be removed by providing NULL to dir */
+    if ( NULL != dir )
+    {
+        monitor = g_file_monitor_directory (
+                dir,
+                G_FILE_MONITOR_NONE,
+                NULL,
+                NULL);
+
+        g_signal_connect (
+                G_OBJECT(monitor),
+                "changed",
+                G_CALLBACK (cb_file_monitor_changed),
+                image_list);
+    }
+
+    image_list->priv->monitor = monitor;
+}
+
+static void
+cb_file_monitor_changed (
+        GFileMonitor      *monitor,
+        GFile             *file,
+        GFile             *other_file,
+        GFileMonitorEvent  event_type,
+        gpointer           user_data )
+{
+    RsttoImageList *image_list = RSTTO_IMAGE_LIST (user_data);
+    RsttoFile *r_file = rstto_file_new (file);
+
+    g_return_if_fail ( monitor == image_list->priv->monitor);
+
+    switch ( event_type )
+    {
+        case G_FILE_MONITOR_EVENT_DELETED:
+            rstto_image_list_remove_file ( image_list, r_file );
+            break;
+        case G_FILE_MONITOR_EVENT_CREATED:
+            rstto_image_list_add_file (image_list, r_file, NULL);
+            break;
+        case G_FILE_MONITOR_EVENT_MOVED:
+            rstto_image_list_remove_file ( image_list, r_file );
+
+            /* Remove our reference, reusing pointer */
+            g_object_unref (r_file);
+
+            r_file = rstto_file_new (other_file);
+            rstto_image_list_add_file (image_list, r_file, NULL);
+            break;
+        default:
+            break;
+    }
+
+    if ( NULL != r_file )
+    {
+        g_object_unref (r_file);
+    }
+}
+
 
 
 GType
@@ -428,6 +647,7 @@ rstto_image_list_iter_find_file (RsttoImageListIter *iter, RsttoFile *file)
             iter->priv->file = NULL;
         }
         iter->priv->file = file;
+        iter->priv->sticky = TRUE;
 
         g_signal_emit (G_OBJECT (iter), rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_CHANGED], 0, NULL);
 
@@ -452,9 +672,11 @@ rstto_image_list_iter_get_file (RsttoImageListIter *iter)
     return iter->priv->file;
 }
 
-
-void
-rstto_image_list_iter_set_position (RsttoImageListIter *iter, gint pos)
+static void
+iter_set_position (
+        RsttoImageListIter *iter,
+        gint pos,
+        gboolean sticky )
 {
     g_signal_emit (G_OBJECT (iter), rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_PREPARE_CHANGE], 0, NULL);
 
@@ -472,11 +694,20 @@ rstto_image_list_iter_set_position (RsttoImageListIter *iter, gint pos)
 }
 
 void
-rstto_image_list_iter_next (RsttoImageListIter *iter)
+rstto_image_list_iter_set_position (RsttoImageListIter *iter, gint pos)
+{
+    iter_set_position ( iter, pos, TRUE );
+}
+
+static gboolean
+iter_next (
+        RsttoImageListIter *iter,
+        gboolean sticky)
 {
     GList *position = NULL;
-    RsttoSettings *settings = NULL;
+    RsttoImageList *image_list = iter->priv->image_list;
     RsttoFile *file = iter->priv->file;
+    gboolean ret_val = FALSE;
 
     g_signal_emit (G_OBJECT (iter), rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_PREPARE_CHANGE], 0, NULL);
 
@@ -486,15 +717,28 @@ rstto_image_list_iter_next (RsttoImageListIter *iter)
         iter->priv->file = NULL;
     }
 
+    iter->priv->sticky = sticky;
+
     position = g_list_next (position);
     if (position)
+    {
         iter->priv->file = position->data; 
+
+        /* We could move forward, set ret_val to TRUE */
+        ret_val = TRUE;
+    }
     else
     {
-        settings = rstto_settings_new();
 
-        if (rstto_settings_get_boolean_property (settings, "wrap-images"))
+        if (TRUE == image_list->priv->wrap_images)
+        {
             position = g_list_first (iter->priv->image_list->priv->images);
+
+            /* We could move forward, wrapped back to the start of the
+             * list, set ret_val to TRUE
+             */
+            ret_val = TRUE;
+        }
         else
             position = g_list_last (iter->priv->image_list->priv->images);
 
@@ -502,22 +746,54 @@ rstto_image_list_iter_next (RsttoImageListIter *iter)
             iter->priv->file = position->data; 
         else
             iter->priv->file = NULL;
-
-        g_object_unref (settings);
     }
 
     if (file != iter->priv->file)
     {
         g_signal_emit (G_OBJECT (iter), rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_CHANGED], 0, NULL);
     }
+
+    return ret_val;
 }
 
-void
-rstto_image_list_iter_previous (RsttoImageListIter *iter)
+gboolean
+rstto_image_list_iter_next (RsttoImageListIter *iter)
+{
+    return iter_next (iter, TRUE);
+}
+
+gboolean
+rstto_image_list_iter_has_next (RsttoImageListIter *iter)
+{
+    RsttoImageList *image_list = iter->priv->image_list;
+
+    if (image_list->priv->wrap_images)
+    {
+        return TRUE;
+    }
+    else
+    {
+        if (rstto_image_list_iter_get_position (iter) ==
+            (rstto_image_list_get_n_images (image_list) -1))
+        {
+            return FALSE;
+        }
+        else
+        {
+            return TRUE;
+        }
+    }
+}
+
+static gboolean
+iter_previous (
+        RsttoImageListIter *iter,
+        gboolean sticky)
 {
     GList *position = NULL;
-    RsttoSettings *settings = NULL;
+    RsttoImageList *image_list = iter->priv->image_list;
     RsttoFile *file = iter->priv->file;
+    gboolean ret_val = FALSE;
 
     g_signal_emit (G_OBJECT (iter), rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_PREPARE_CHANGE], 0, NULL);
 
@@ -527,6 +803,8 @@ rstto_image_list_iter_previous (RsttoImageListIter *iter)
         iter->priv->file = NULL;
     }
 
+    iter->priv->sticky = sticky;
+
     position = g_list_previous (position);
     if (position)
     {
@@ -534,26 +812,62 @@ rstto_image_list_iter_previous (RsttoImageListIter *iter)
     }
     else
     {
-        settings = rstto_settings_new();
-
-        if (rstto_settings_get_boolean_property (settings, "wrap-images"))
+        if (TRUE == image_list->priv->wrap_images)
+        {
             position = g_list_last (iter->priv->image_list->priv->images);
+        }
         else
+        {
             position = g_list_first (iter->priv->image_list->priv->images);
+        }
 
         if (position)
+        {
             iter->priv->file = position->data; 
+        }
         else
+        {
             iter->priv->file = NULL;
-
-        g_object_unref (settings);
+        }
     }
 
     if (file != iter->priv->file)
     {
         g_signal_emit (G_OBJECT (iter), rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_CHANGED], 0, NULL);
     }
+
+    return ret_val;
+
 }
+gboolean
+rstto_image_list_iter_previous (RsttoImageListIter *iter)
+{
+    return iter_previous (iter, TRUE);
+}
+
+
+gboolean
+rstto_image_list_iter_has_previous (RsttoImageListIter *iter)
+{
+    RsttoImageList *image_list = iter->priv->image_list;
+
+    if (image_list->priv->wrap_images)
+    {
+        return TRUE;
+    }
+    else
+    {
+        if (rstto_image_list_iter_get_position (iter) == 0)
+        {
+            return FALSE;
+        }
+        else
+        {
+            return TRUE;
+        }
+    }
+}
+
 
 RsttoImageListIter *
 rstto_image_list_iter_clone (RsttoImageListIter *iter)
@@ -717,3 +1031,30 @@ cb_rstto_image_list_exif_date_compare_func (RsttoFile *a, RsttoFile *b)
     }
     return 1;
 }
+
+gboolean
+rstto_image_list_iter_get_sticky (
+        RsttoImageListIter *iter)
+{
+    return iter->priv->sticky;
+}
+
+static void
+cb_rstto_wrap_images_changed (
+        GObject *settings,
+        GParamSpec *pspec,
+        gpointer user_data)
+{
+    GValue val_wrap_images = { 0, };
+
+    RsttoImageList *image_list = RSTTO_IMAGE_LIST (user_data);
+
+    g_value_init (&val_wrap_images, G_TYPE_BOOLEAN);
+
+    g_object_get_property (
+            settings,
+            "wrap-images",
+            &val_wrap_images);
+
+    image_list->priv->wrap_images = g_value_get_boolean (&val_wrap_images);
+}
