@@ -21,6 +21,7 @@
 #include <gtk/gtkmarshal.h>
 #include <string.h>
 #include <gio/gio.h>
+#include <libxfce4ui/libxfce4ui.h>
 #include <libexif/exif-data.h>
 
 #include <math.h>
@@ -68,10 +69,13 @@ struct _RsttoImageViewerPriv
     GdkColormap                 *colormap;
 
     GtkIconTheme                *icon_theme;
+    GdkPixbuf                   *missing_icon;
     GdkPixbuf                   *bg_icon;
     GdkColor                    *bg_color;
     GdkColor                    *bg_color_fs;
 
+    GError                      *error;
+
     RsttoImageViewerTransaction *transaction;
     GdkPixbuf                   *pixbuf;
     RsttoImageOrientation        orientation;
@@ -124,6 +128,8 @@ struct _RsttoImageViewerTransaction
     GCancellable     *cancellable;
     GdkPixbufLoader  *loader;
 
+    GError           *error;
+
     gint              image_width;
     gint              image_height;
     gdouble           image_scale;
@@ -289,6 +295,12 @@ rstto_image_viewer_init ( GObject *object )
             BACKGROUND_ICON_SIZE,
             0,
             NULL);
+    viewer->priv->missing_icon = gtk_icon_theme_load_icon (
+            viewer->priv->icon_theme,
+            "image-missing",
+            BACKGROUND_ICON_SIZE,
+            0,
+            NULL);
     gdk_pixbuf_saturate_and_pixelate (
             viewer->priv->bg_icon,
             viewer->priv->bg_icon,
@@ -573,6 +585,11 @@ rstto_image_viewer_destroy(GtkObject *object)
             g_object_unref (viewer->priv->bg_icon);
             viewer->priv->bg_icon = NULL;
         }
+        if (viewer->priv->missing_icon)
+        {
+            g_object_unref (viewer->priv->missing_icon);
+            viewer->priv->missing_icon = NULL;
+        }
         if (viewer->priv->pixbuf)
         {
             g_object_unref (viewer->priv->pixbuf);
@@ -957,6 +974,7 @@ paint_image (
     gdouble y_offset;
     gint block_width = 10;
     gint block_height = 10;
+    gdouble bg_scale = 1.0;
 
     if (viewer->priv->pixbuf)
     {
@@ -1137,6 +1155,53 @@ paint_image (
                 0.0);
         cairo_paint (ctx);
     }
+    else
+    {
+        if (viewer->priv->error)
+        {
+            /* Calculate the icon-size */
+            /***************************/
+            if (widget->allocation.width < widget->allocation.height)
+            {
+                bg_scale = (gdouble)BACKGROUND_ICON_SIZE /
+                    (gdouble)widget->allocation.width * 1.2;
+            }
+            else
+            {
+                bg_scale = (gdouble)BACKGROUND_ICON_SIZE /
+                    (gdouble)widget->allocation.height * 1.2;
+            }
+
+            /* Move the cairo context in position so the
+             * background-image is painted in the center
+             * of the widget.
+             */
+            cairo_translate (
+                    ctx,
+                    (gdouble)(widget->allocation.width-BACKGROUND_ICON_SIZE/bg_scale)/2.0,
+                    (gdouble)(widget->allocation.height-BACKGROUND_ICON_SIZE/bg_scale)/2.0);
+
+            /* Scale the context so the image
+             * fills the same part of the cairo-context
+             */
+            cairo_scale (
+                    ctx,
+                    1.0 / bg_scale,
+                    1.0 / bg_scale);
+
+            /* Draw the pixbuf on the cairo-context */
+            /****************************************/
+            if(viewer->priv->missing_icon != NULL)
+            {
+                gdk_cairo_set_source_pixbuf (
+                        ctx,
+                        viewer->priv->missing_icon,
+                        0.0,
+                        0.0);
+                cairo_paint_with_alpha (ctx, 1.0);
+            }
+        }
+    }
 
 }
 
@@ -1357,6 +1422,14 @@ rstto_image_viewer_set_file (
                 g_object_unref (viewer->priv->file);
 
                 viewer->priv->file = file;
+                if (viewer->priv->error)
+                {
+                    g_error_free (viewer->priv->error);
+                    viewer->priv->error = NULL;
+                }
+                viewer->priv->image_scale = 1.0;
+                viewer->priv->image_width = 1.0;
+                viewer->priv->image_height = 1.0;
 
                 rstto_image_viewer_load_image (
                         viewer,
@@ -1481,6 +1554,10 @@ rstto_image_viewer_transaction_free (RsttoImageViewerTransaction *tr)
     {
         tr->viewer->priv->transaction = NULL;
     }
+    if (tr->error)
+    {
+        g_error_free (tr->error);
+    }
     g_object_unref (tr->cancellable);
     g_object_unref (tr->loader);
     g_free (tr->buffer);
@@ -1695,9 +1772,8 @@ cb_rstto_image_viewer_read_input_stream_ready (
         GAsyncResult *result,
         gpointer user_data )
 {
-    GError *error = NULL;
     RsttoImageViewerTransaction *transaction = (RsttoImageViewerTransaction *)user_data;
-    gssize read_bytes = g_input_stream_read_finish (G_INPUT_STREAM (source_object), result, &error);
+    gssize read_bytes = g_input_stream_read_finish (G_INPUT_STREAM (source_object), result, &transaction->error);
 
     if (read_bytes == -1)
     {
@@ -1707,7 +1783,7 @@ cb_rstto_image_viewer_read_input_stream_ready (
 
     if (read_bytes > 0)
     {
-        if(gdk_pixbuf_loader_write (transaction->loader, (const guchar *)transaction->buffer, read_bytes, &error) == FALSE)
+        if(gdk_pixbuf_loader_write (transaction->loader, (const guchar *)transaction->buffer, read_bytes, &transaction->error) == FALSE)
         {
             /* Clean up the input-stream */
             g_input_stream_close (G_INPUT_STREAM (source_object), NULL, NULL);
@@ -1726,7 +1802,7 @@ cb_rstto_image_viewer_read_input_stream_ready (
     }
     else {
         /* Loading complete, transaction should not be free-ed */
-        gdk_pixbuf_loader_close (transaction->loader, NULL);
+        gdk_pixbuf_loader_close (transaction->loader, &transaction->error);
 
         /* Clean up the input-stream */
         g_input_stream_close (G_INPUT_STREAM (source_object), NULL, NULL);
@@ -1845,13 +1921,32 @@ cb_rstto_image_loader_closed (GdkPixbufLoader *loader, RsttoImageViewerTransacti
     if (viewer->priv->transaction == transaction)
     {
         
-        viewer->priv->image_scale = transaction->image_scale;
-        viewer->priv->image_width = transaction->image_width;
-        viewer->priv->image_height = transaction->image_height;
-        viewer->priv->orientation = transaction->orientation;
+        if (NULL == transaction->error)
+        {
+            gtk_widget_set_tooltip_text (GTK_WIDGET (viewer), NULL);
+            viewer->priv->image_scale = transaction->image_scale;
+            viewer->priv->image_width = transaction->image_width;
+            viewer->priv->image_height = transaction->image_height;
+            viewer->priv->orientation = transaction->orientation;
+            set_scale (viewer, transaction->scale);
+        }
+        else
+        {
+            viewer->priv->image_scale = 1.0;
+            viewer->priv->image_width = 1.0;
+            viewer->priv->image_height = 1.0;
+            if (viewer->priv->pixbuf)
+            {
+                g_object_unref (viewer->priv->pixbuf);
+                viewer->priv->pixbuf = NULL;
+            }
 
-        set_scale (viewer, transaction->scale);
+            gtk_widget_set_tooltip_text (GTK_WIDGET (viewer), transaction->error->message);
+        }
 
+
+        viewer->priv->error = transaction->error;
+        transaction->error = NULL;
         viewer->priv->transaction = NULL;
 
         gdk_window_invalidate_rect (
@@ -2530,3 +2625,13 @@ cb_rstto_image_viewer_dnd (GtkWidget *widget, GdkDragContext *context, gint x, g
         gtk_drag_finish (context, FALSE, FALSE, time_);
     }
 }
+
+GError *
+rstto_image_viewer_get_error ( RsttoImageViewer *viewer )
+{
+    if (viewer->priv->error)
+    {
+        return g_error_copy (viewer->priv->error);
+    }
+    return NULL;
+}
